export interface IUnifiedRequest {
    method?: 'get' | 'post' | 'patch' | 'put' | 'delete' | 'head';
    baseUrl?: string;
    url: string;
    queries?: {
        [key: string]: string;
    };
    parameters?: {
        [key: string]: string;
    };
    headers?: {
        [key: string]: string;
    };
    body?: any;
    processor?: IUnifiedRequestProcessor;
    [optionalProp: string]: unknown;
}
export interface IUnifiedRequestProcessorConfig {
    method: 'get' | 'post' | 'patch' | 'put' | 'delete' | 'head';
    url: string;
    headers?: {
        [key: string]: string;
    };
    body?: any;
    [optionalProp: string]: unknown;
}
export interface IUnifiedResponse {
    status: number;
    headers: {
        [key: string]: string;
    };
    data?: any;
}
export declare type IUnifiedRequestProcessor = ((config: IUnifiedRequestProcessorConfig) => Promise<IUnifiedResponse>);
export declare function httpProcessorFetch({ method, url, headers, body, }: IUnifiedRequestProcessorConfig): Promise<{
    status: number;
    headers: Record<string, string>;
    data: any;
}>;
export declare class UnifiedNetwork {
    base: Partial<IUnifiedRequest>;
    constructor(base?: Partial<IUnifiedRequest>);
    request(config: IUnifiedRequest): Promise<{
        status: number;
        data: any;
        headers: Record<string, string>;
    }>;
    setHeader(header: string, value: string): void;
    removeHeader(header: string): void;
    applyHeader(header: string, value: string): void;
}
