"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnifiedNetwork = exports.httpProcessorFetch = void 0;
const util_1 = require("./util");
async function httpProcessorFetch({ method, url, headers, body, }) {
    var _a;
    let responseStatus = undefined;
    let responseData = undefined;
    let responseHeaders = undefined;
    const response = await fetch(url, {
        method,
        body,
        headers,
    });
    responseStatus = response.status;
    responseHeaders = Object.fromEntries(response.headers.entries());
    if (response.ok) {
        responseData = await response.text();
        if ((_a = responseHeaders['content-type']) === null || _a === void 0 ? void 0 : _a.toLowerCase().includes('application/json')) {
            try {
                responseData = JSON.parse(responseData);
            }
            catch (error) {
                throw new Error('could not parse response data ' + error.message);
            }
        }
    }
    else {
        responseData = await response.text();
    }
    return {
        status: responseStatus,
        headers: responseHeaders,
        data: responseData
    };
}
exports.httpProcessorFetch = httpProcessorFetch;
class UnifiedNetwork {
    constructor(base) {
        var _a;
        var _b;
        this.base = base !== null && base !== void 0 ? base : {};
        (_a = (_b = this.base).headers) !== null && _a !== void 0 ? _a : (_b.headers = {});
    }
    async request(config) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
        const method = (_a = config.method) !== null && _a !== void 0 ? _a : this.base.method;
        const baseUrl = (_b = config.baseUrl) !== null && _b !== void 0 ? _b : this.base.baseUrl;
        const url = (_d = (_c = config.url) !== null && _c !== void 0 ? _c : this.base.url) !== null && _d !== void 0 ? _d : '/';
        const parameters = Object.assign(Object.assign({}, ((_e = this.base.parameters) !== null && _e !== void 0 ? _e : {})), ((_f = config.parameters) !== null && _f !== void 0 ? _f : {}));
        const queries = Object.assign(Object.assign({}, ((_g = this.base.queries) !== null && _g !== void 0 ? _g : {})), ((_h = config.queries) !== null && _h !== void 0 ? _h : {}));
        let body = (_j = config.body) !== null && _j !== void 0 ? _j : this.base.body;
        const processor = (_l = (_k = config.processor) !== null && _k !== void 0 ? _k : this.base.processor) !== null && _l !== void 0 ? _l : httpProcessorFetch;
        const otherProps = Object.assign(Object.assign({}, this.base), config);
        delete otherProps['method'];
        delete otherProps['baseUrl'];
        delete otherProps['url'];
        delete otherProps['parameters'];
        delete otherProps['queries'];
        delete otherProps['body'];
        delete otherProps['processor'];
        let fullUrl = baseUrl ? (0, util_1.joinPaths)(baseUrl, url) : url;
        if (parameters && Object.keys(parameters).length > 0) {
            for (const key in parameters) {
                fullUrl = fullUrl.replaceAll(`[${key}]`, parameters[key]);
            }
        }
        if (queries && Object.keys(queries).length > 0) {
            fullUrl = fullUrl + (fullUrl.includes('?') ? '&' : '?') + Object.keys(queries).map(key => `${key}=${queries[key]}`).join('&');
        }
        let headers = Object.assign(Object.assign({}, ((_o = (_m = this.base) === null || _m === void 0 ? void 0 : _m.headers) !== null && _o !== void 0 ? _o : {})), ((_p = config.headers) !== null && _p !== void 0 ? _p : {}));
        if (typeof body === 'object' && body !== null && !(body instanceof FormData) && !(body instanceof URLSearchParams)) {
            body = JSON.stringify(body);
            headers['Content-Type'] = 'application/json';
        }
        let responseStatus = undefined;
        let responseData = undefined;
        let responseHeaders = {};
        try {
            const { status: processorStatus, data: processorData, headers: processorHeaders } = await processor(Object.assign(Object.assign({}, otherProps), { url: fullUrl, method: method !== null && method !== void 0 ? method : 'get', headers,
                body }));
            responseStatus = processorStatus;
            responseHeaders = processorHeaders;
            responseData = processorData;
        }
        catch (error) {
            console.error(error);
            responseStatus = -1;
            responseData = undefined;
        }
        return {
            status: responseStatus,
            data: responseData,
            headers: responseHeaders
        };
    }
    setHeader(header, value) {
        this.base.headers[header] = value;
    }
    removeHeader(header) {
        delete this.base.headers[header];
    }
    applyHeader(header, value) {
        if (value === undefined || value == null || value === '') {
            this.removeHeader(header);
        }
        else {
            this.setHeader(header, value);
        }
    }
}
exports.UnifiedNetwork = UnifiedNetwork;
